@@ -1,6 +1,7 @@
 #ifndef LIBFILEZILLA_FILE_HEADER
 #define LIBFILEZILLA_FILE_HEADER
 
+#include "fsresult.hpp"
 #include "libfilezilla.hpp"
 
 #ifdef FZ_WINDOWS
@@ -15,6 +16,8 @@
 
 namespace fz {
 
+class datetime;
+
 /** \brief Lean class for file access
  *
  * This class uses the system's native file access functions. It is a less convoluted and much faster alternative
@@ -25,10 +28,17 @@ namespace fz {
 class FZ_PUBLIC_SYMBOL file final
 {
 public:
-	/// Files can be opened for reading or writing, but not both
+#ifdef FZ_WINDOWS
+	typedef HANDLE file_t;
+#else
+	typedef int file_t;
+#endif
+
+	/// Files can be opened for reading, writing, or both
 	enum mode {
 		reading,
-		writing
+		writing,
+		readwrite
 	};
 
 	/** \brief Creation flags when opening file for writing.
@@ -68,6 +78,13 @@ public:
 	file() = default;
 	file(native_string const& f, mode m, creation_flags d = existing);
 
+
+	/** \brief Creates file from descriptor
+	 *
+	 * Takes ownership of descriptor/handle.
+	 */
+	explicit file(file_t fd);
+
 	~file();
 
 	file(file const&) = delete;
@@ -79,10 +96,17 @@ public:
 	bool opened() const;
 	explicit operator bool() const { return opened(); }
 
-	bool open(native_string const& f, mode m, creation_flags d = existing);
+	result open(native_string const& f, mode m, creation_flags d = existing);
 
 	void close();
 
+	/// Returns the raw file descriptor, but retains ownership.
+	file_t fd() {
+		return fd_;
+	}
+
+	file_t detach();
+
 	/// Used by \ref seek
 	enum seek_mode {
 		/// Seek from beginning of file
@@ -158,9 +182,15 @@ public:
 	 */
 	bool fsync();
 
+	/** \brief Sets modification time to specified time.
+	 *
+	 * File must be opened for writing, of the call will fail.
+	 */
+	bool set_modification_time(datetime const& t);
+
 private:
 #ifdef FZ_WINDOWS
-	HANDLE hFile_{INVALID_HANDLE_VALUE};
+	HANDLE fd_{INVALID_HANDLE_VALUE};
 #else
 	int fd_{-1};
 #endif
@@ -175,6 +205,10 @@ bool FZ_PUBLIC_SYMBOL remove_file(native_string const& name);
 inline file::creation_flags operator|(file::creation_flags lhs, file::creation_flags rhs) {
 	return static_cast<file::creation_flags>(static_cast<unsigned int>(lhs) | rhs);
 }
+inline file::creation_flags& operator|=(file::creation_flags & lhs, file::creation_flags rhs) {
+	lhs = static_cast<file::creation_flags>(static_cast<unsigned int>(lhs) | rhs);
+	return lhs;
+}
 
 }
 #endif
